@@ -1,15 +1,23 @@
 import functools
-import numpy as np
-import k3d
-from k3d.objects import Group, Line, Points
-from k3d.plot import Plot
-from .jointset import JointSet, MuPoTSJoints, OpenPoseJoints, CocoExJoints, PanopticJoints, Common14Joints
-from typing import Union, NewType
+from typing import Union, NewType, Optional, List, Dict, Final
 
-SkeletonPlot = NewType('SkeletonPlot', Union[Plot, None])
-SkeletonObject = NewType('SkeletonObject', Union[Group, None])
+import k3d
+import numpy as np
+from k3d.objects import Group, Line, Points, Text, Drawable
+from k3d.plot import Plot
+
+from .jointset import JointSet, MuPoTSJoints, OpenPoseJoints, CocoExJoints, PanopticJoints, Common14Joints
+
+SkeletonPlot = NewType('SkeletonPlot', Optional[Plot])
+SkeletonObject = NewType('SkeletonObject', Optional[Group])
 Color = NewType('Color', Union[str, int])
 
+DEFAULT_COLORS: Final[Dict[str, int]] = {
+    'red': 0xFF0000, 'green': 0x00FF00,  'blue': 0x0000FF,
+    'yellow': 0xFFFF00, 'teal': 0x00FFFF, 'purple': 0xFF00FF,
+    'white': 0xFFFFFF, 'black': 0x000000
+}
+
 
 class Skeleton:
     def __init__(self, joint_coordinates: np.ndarray,
@@ -21,13 +29,14 @@ class Skeleton:
 
     def to_skeleton_object(self) -> SkeletonObject:
         joint_points = self.get_joint_points()
-        joint_lines = self.get_joint_lines()
-        skeleton_object: SkeletonObject = SkeletonObject()
+        joint_lines: List[Drawable] = self.get_joint_lines()
+        joint_names: List[Drawable] = self.get_joint_names()
+        skeleton_object: SkeletonObject = SkeletonObject(Group())
         skeleton_object += joint_points
         return functools.reduce(
-            function=lambda _skeleton_object, _skeleton_part: _skeleton_object + _skeleton_part,
-            sequence=joint_lines,
-            initial=skeleton_object
+            lambda _skeleton_object, _skeleton_part: _skeleton_object + _skeleton_part,
+            joint_lines + joint_names,
+            skeleton_object
         )
 
     def get_joint_colors(self) -> np.ndarray:
@@ -51,8 +60,8 @@ class Skeleton:
             line_colors[self.joint_set.center_line_indices] = 0xFFFFFF
             return line_colors
         else:
-            np.full(shape=line_colors_shape,
-                    fill_value=self.color, dtype='uint32')
+            return np.full(shape=line_colors_shape,
+                           fill_value=self.color, dtype='uint32')
 
     def get_joint_points(self) -> Points:
         skeleton_joint_colors = self.get_joint_colors()
@@ -61,14 +70,22 @@ class Skeleton:
             shader='mesh', colors=skeleton_joint_colors
         )
 
-    def get_joint_lines(self) -> list[Line]:
+    def get_joint_lines(self) -> List[Line]:
         skeleton_line_colors = self.get_line_colors()
         return [self.__create_line_between_joints(
             start=self.joint_coordinates[self.joint_set.limb_graph[i][0]],
             end=self.joint_coordinates[self.joint_set.limb_graph[i][1]],
             width=self.part_size / 4.0,
-            color=int(skeleton_line_colors[i])
-        ) for i in range(len(self.joint_set.limb_graph))]
+            color=int(skeleton_line_colors[i]))
+            for i in range(len(self.joint_set.limb_graph))
+        ]
+
+    def get_joint_names(self) -> List[Text]:
+        return [k3d.text(
+            text=self.joint_set.names[i],
+            position=self.joint_coordinates[i])
+            for i in range(self.joint_set.number_of_joints)
+        ]
 
     @staticmethod
     def __create_line_between_joints(start: float, end: float, width: float, color: int) -> Line:
@@ -84,7 +101,7 @@ class SkeletonVisualizer:
         self.plot: SkeletonPlot = None
         self.size_scalar: float = size_scalar
 
-    def visualize(self, skeletons: np.ndarray, colors: list[Color] = None):
+    def visualize(self, skeletons: np.ndarray, colors: List[Color] = None):
         self.__assert_visualization_arguments(skeletons, colors)
         number_of_skeletons = skeletons.shape[0]
         if colors is None:
@@ -92,26 +109,27 @@ class SkeletonVisualizer:
         self.plot = self.__create_skeleton_plot(skeletons, colors)
         self.plot.display()
 
-    def __assert_visualization_arguments(self, skeletons: np.ndarray, colors: list[Color]):
+    def __assert_visualization_arguments(self, skeletons: np.ndarray, colors: List[Color]):
         assert len(skeletons.shape) == 3, 'The \'skeletons\' parameter should be a 3 dimensional numpy array.'
         if colors is not None:
-            assert skeletons.shape[0] == len(colors)
-        assert skeletons.shape[1] == self.joint_set.number_of_joints
-        assert skeletons.shape[2] == 3
+            assert skeletons.shape[0] == len(colors),\
+                'The \'skeletons\' and \'colors\' parameters must be the same length.'
+        assert skeletons.shape[1] == self.joint_set.number_of_joints,\
+            'The number of joints of skeletons and the number of joints in the specified joint set must be the same.'
+        assert skeletons.shape[2] == 3, 'The skeleton joint coordinates must be 3 dimensional'
 
-    def __create_skeleton_plot(self, skeletons: np.ndarray, colors: list[Color]) -> SkeletonPlot:
+    def __create_skeleton_plot(self, skeletons: np.ndarray, colors: List[Color]) -> SkeletonPlot:
         skeleton_part_size = self.__calculate_skeleton_part_size(skeletons)
-        return functools.reduce(
-            function=lambda skeleton_plot, skeleton: skeleton_plot + skeleton.to_skeleton_object(),
-            sequence=map(
-                lambda skeleton, color: Skeleton(
-                    joint_coordinates=skeleton,
+        skeleton_plot = k3d.plot()
+        for skeleton in map(
+                lambda skeleton_color_tuple: Skeleton(
+                    joint_coordinates=skeleton_color_tuple[0],
                     joint_set=self.joint_set,
                     part_size=skeleton_part_size,
-                    color=color),
-                zip(skeletons, colors)),
-            initial=k3d.plot()
-        )
+                    color=skeleton_color_tuple[1]),
+                zip(skeletons, colors)):
+            skeleton_plot += skeleton.to_skeleton_object()
+        return skeleton_plot
 
     def __calculate_skeleton_part_size(self, skeletons: np.ndarray) -> float:
         max_values = [abs(skeleton).max() for skeleton in skeletons]
